﻿using System;
using Lucene.Net.Analysis;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.QueryParsers;
using Lucene.Net.Search;
using Lucene.Net.Store;

namespace Example.LuceneTest
{
    public class Program
    {
        static void Main(string[] args)
        {
            Directory directory = new RAMDirectory();
            Analyzer analyzer = new StandardAnalyzer();

            WriteExampleDocumentsToDirectory(directory, analyzer);

            Hits hits = SearchForExampleText(directory, analyzer);

            WriteSearchResultsToConsole(hits);
        }

        private static void WriteExampleDocumentsToDirectory(Directory directory, Analyzer analyzer)
        {
            IndexWriter indexWriter = new IndexWriter(directory, analyzer, true);

            Document document = CreateDocument();
            indexWriter.AddDocument(document);

            Document secondDocument = CreateSecondDocument();
            indexWriter.AddDocument(secondDocument);

            indexWriter.Close();
        }

        private static Document CreateDocument()
        {
            Document document = new Document();
            
            string blogEntryBody = "This is some example text for the first blog entry body.";
            Field bodyField = new Field("blogEntryBody", blogEntryBody, Field.Store.YES, Field.Index.TOKENIZED);
            document.Add(bodyField);
            return document;
        }

        private static Document CreateSecondDocument()
        {
            Document document = new Document();

            string blogEntryBody = "This is some example text for the second blog entry body. The body of this blog entry is a bit longer than the first.";
            Field bodyField = new Field("blogEntryBody", blogEntryBody, Field.Store.YES, Field.Index.TOKENIZED);
            document.Add(bodyField);
            return document;
        }

        private static Hits SearchForExampleText(Directory directory, Analyzer analyzer)
        {
            IndexSearcher indexSearcher = new IndexSearcher(directory);

            QueryParser queryParser = new QueryParser("blogEntryBody", analyzer);
            
            Query query = queryParser.Parse("example");
            
            return indexSearcher.Search(query);
        }

        private static void WriteSearchResultsToConsole(Hits hits)
        {
            int numberOfResults = hits.Length();
            string numberOfResultsHeader = string.Format("The search returned {0} results.", numberOfResults);
            Console.WriteLine(numberOfResultsHeader);
            
            for (int i = 0; i < hits.Length(); i++)
            {
                float score = hits.Score(i);

                string hitHeader = string.Format("\nHit number {0}, with a score of {1}:", i, score);

                Console.WriteLine(hitHeader);
                Console.WriteLine(hits.Doc(i).Get("blogEntryBody"));
            }
        }
    }
}
